#!/bin/bash
#
# script name:  run_ip.sh
# purpose:      changing route table.
# author:       netman(netman@study-area.org)
# license:      GPL(http://www.gnu.org/licenses/gpl.html)
#
# date:         2004-09-23
# version:      v0.05
#
# caveate:
#       1) tested on redhat 9.0 linux only
#       2) iproute program is required
#
#------------------------------------------------------------
# change log:
#       1) 2004-03-23 v0.01
#               * first beta release
#       2) 2004-03-24 v0.02
#               * BUGFIXED:
#                 - Add IPIF and GWIP checking before change route.
#       3) 2004-04-15 v0.03
#               * Add function:
#                 - Restart network if dead device detected.
#       4) 2004-07-07 v0.04
#               * IMPROVEMENT
#                 - Create central config file
#               * Add function:
#                 - Enable static routing
#       5) 2004-09-23 v0.05
#               * IMPROVEMENT
#                 - More accurate on deleting old rules
#                 - More accurate on deleting dead interface
#		  - Add gw determining for dhcp interface
#		  - Add deletion for dead route
#		* BugFix
#		  - Use central conf file
#		  - Filter out IPv6 address
#
#------------------------------------------------------------


WD=${0%/*}
CONFIG=$WD/configs
[ -f $CONFIG ] && . $CONFIG || {
        echo "${0#*/}: ERROR: can not load $CONFIG."
        exit 1
}

#-- route command prefix --#
DEFGW='/sbin/ip route replace default'

#-- define dead interface --#
while getopts "d:" opt; do
        case $opt in
                d) dead_dev=$OPTARG ;;
        esac
done

#-- determine active interfaces --#
interfaces=$($IP address | $GREP -E 'ppp|eth' | \
	$AWK -F': ' '/^[0-9][0-9]*:/{print $2}' | $SORT -u \
	| grep -Ev "${int_if// /|}")

#-- remove old rule table --#
for i in $interfaces; do
	$IP address | $GREP -A2 $i | $AWK '/inet[^6]/{print $2}' | $CUT -d'/' -f1
done | xargs -n1 echo from | while read line; do
	$IP rule | grep "$line"
done | $SED -e "s/^/ip rule del pref /;s/://;s/lookup/table/" | $BASH
unset i


#-- remove dead interfaces --#
if [ -n "$dead_dev" ]; then
        interfaces=$(echo "$interfaces" | grep -Ev "${dead_dev// /|}")
	for dev in $dead_dev; do
		DEF_ROUTE=$($IP route list | grep '^default' | grep "dev $dev")
		echo "$DEF_ROUTE" | while read line; do
			$IP route del $line
		done
	done
fi

#-- exit while all dead --#
[ "$interfaces" ] || {
	$IP route del default
	exit 2
}


#-- define table_id --#
init_num=10
offset=`echo -e "${interfaces// /\n}" | $WC -l`
last_num=$(($init_num + $((offset * 10)) ))
tb_num=`$SEQ -s ' ' $init_num 10 $last_num`

#-- FUNCTION: determine DHCP assigning gw --#
dhgw() {
    	get_lastest () {
		OIFS="$IFS"
		IFS='
'	
		for line in $RnT1 $RNT2; do
			l="$(echo $line | grep -o 'lease-time[^;]*' | cut -d' ' -f2)"
			r="$(echo $line | grep -o 'routers[^;]*' | cut -d' ' -f2)"
			e="$(echo $line | grep -o 'expire[^;]*' | cut -d' ' -f3-)"
			EX_TIME=$($DATE -d "$e" +%s)
			if [ "$((EX_TIME - l))" -gt "${P_TIME-0}" ]; then
				P_TIME=$((EX_TIME-l))
				D_TIME=$EX_TIME
				ROUTER=$r
			fi
		done
		IFS="$OIFS"
		echo $D_TIME $ROUTER
    	}

	DH_IF=$1
	shift 1
	RnT1=$($CAT $1 2>/dev/null | xargs echo | tr '}{' '\n' | $GREP $DH_IF | $TAIL -n 2)
	RnT2=$($CAT $1 2>/dev/null | xargs echo | tr '}{' '\n' | $GREP $DH_IF | $TAIL -n 2)
	[ "$RnT1" -o "$RnT2" ] || return
	GnT=$(get_lastest)
	C_TIME=$($DATE +%s)
	[ "${GnT% *}" -gt $C_TIME ] && GWIP=${GnT#* }
}


#-- FUNCTION: determine gw --#
dgw() {
	unset GWIP
        #-- for fixed if --#
        if echo $fixed_if | $GREP -wq $1 ; then
                GWIP=`echo -e "${fixed_gw// /\n}" | $GREP "^$1:" | \
                        $CUT -d: -f2`
                WEIGHT=`echo -e "${fixed_gw// /\n}" | $GREP "^$1:" | \
                        $CUT -d: -f3`
        #-- for ppp if --#
        elif echo $1 | $GREP -q ppp ; then
                GWIP="`$IP address show dev $1 | $AWK '/inet[^6]/{print $4}' | \
                        $CUT -d'/' -f1 | $HEAD -n 1`"
	else
	#-- try current gw --#
		GWIP=$($ROUTE -n | $GREP $1'$' | $AWK '/^0.0.0.0/{print $2}' | $HEAD -n 1)
	fi
	#-- for dhcp if --#
	[ "$GWIP" ] || dhgw $1 $DHCP_DIR/dhclient.leases $DHCP_DIR/dhclient-$i.leases
}

#-- config rule & routing --#
for i in $interfaces ; do
        unset IFIP GWIP WEIGHT
        IFIP="`$IP address show dev $i | $AWK '/inet[^6]/{print $2}' | \
                $CUT -d'/' -f1`"
        dgw $i
        [ "$tb_num" ] && tb_id=${tb_num%% *}
        [ "$IFIP" -a "$GWIP" ] || continue
	# check gw
        $IP route replace $fixed_dest via $GWIP dev $i &>/dev/null && \
        	sleep 2; $PING -c1 -w2 $fixed_dest &>/dev/null && GWOK=true
        $IP route del $fixed_dest via $GWIP dev $i &>/dev/null
	if [ "$GWOK" = true ]; then
		unset GWOK
	else
		continue
	fi
        # set rule
        $IP rule add pref $tb_id from $IFIP table $tb_id
        # set route to gw
        $IP route replace default via $GWIP dev $i table $tb_id
        DEFGW=$DEFGW" nexthop via $GWIP dev $i weight ${WEIGHT:=1}"
	SETGW=true
        tb_num=${tb_num#* }
done

#-- apply routing --#
[ "$SETGW" = true ] && {
	#-- remove old default route --#
	DEF_NU=$($IP route | $GREP -c '^default')
	for ((i=1;i<=$DEF_NU;i++)); do
		$IP route del default
	done
	#-- set new default route --#
	$DEFGW
}

#-- apply static routes  --#
[ -f $STATIC ] && . $STATIC

#-- flush cache --#
$IP route flush cache

#-- show routing
$IP route list

#-- FINISH --#
exit 0

